@@ -1,7 +1,8 @@
 
 package PRANG;
+use PRANG::Marshaller;
 
-our $VERSION = "0.11_02";
+our $VERSION = "0.01";
 
 1;
 
@@ -17,6 +18,7 @@ PRANG - XML graph engine - XML to Moose objects and back!
  package XML::Language::Node;
  use Moose::Role;
  sub xmlns { "http://example.com/language/1.0" }
+ with 'PRANG::Graph::Class';
 
  # step 2. define the root node(s) of your language
  package XML::Language;
@@ -87,8 +89,7 @@ PRANG - XML graph engine - XML to Moose objects and back!
      );
 
  # step 4b.  emit!
- $format = 1;
- print $object->to_xml($format);
+ print $object->to_xml;
 
 =head1 DESCRIPTION
 
@@ -153,7 +154,7 @@ release, once the "parse" role method is made to work correctly.
 =item B<marshall XML out>
 
 A L<PRANG::Graph> structure also has a C<to_xml> method, which emits
-XML (optionally indented).
+XML.
 
 =back
 
