@@ -11,24 +11,12 @@ has 'xmlns' =>
 	predicate => "has_xmlns",
 	;
 
-has 'xmlns_attr' =>
-	is => "rw",
-	isa => "Str",
-	predicate => "has_xmlns_attr",
-	;
-
 has 'xml_nodeName' =>
 	is => "rw",
 	isa => "Str|HashRef",
 	predicate => "has_xml_nodeName",
 	;
 
-has 'xml_nodeName_prefix' =>
-	is => "rw",
-	isa => "HashRef[Str]",
-	predicate => "has_xml_nodeName_prefix",
-	;
-
 has 'xml_nodeName_attr' =>
 	is => "rw",
 	isa => "Str",
@@ -53,11 +41,9 @@ has 'xml_max' =>
 	predicate => "has_xml_max",
 	;
 
-# FIXME: see commitlog, core Moose should get support for this again
-#        (perhaps)
-#has '+isa' =>
-#	required => 1,
-#	;
+has '+isa' =>
+	required => 1,
+	;
 
 has 'graph_node' =>
 	is => "rw",
@@ -73,7 +59,7 @@ has 'graph_node' =>
 use constant HIGHER_ORDER_TYPE =>
 	"Moose::Meta::TypeConstraint::Parameterized";
 
-method _error(Str $message) {
+method error(Str $message) {
 	my $class = $self->associated_class;
 	my $context = " (Element: ";
 	if ( $class ) {
@@ -83,15 +69,7 @@ method _error(Str $message) {
 		$context .= "(unassociated)";
 	}
 	$context .= "/".$self->name.") ";
-	$message.$context;
-}
-
-method error(Str $message) {
-	die $self->_error($message);
-}
-
-method warn_of(Str $message) {
-	warn $self->_error($message)."\n";
+	die $message.$context;
 }
 
 method build_graph_node() {
@@ -100,18 +78,14 @@ method build_graph_node() {
 	if ( $self->has_xml_required ) {
 		$expect_one = $self->xml_required;
 	}
-	elsif ( $self->has_predicate or
-			$self->has_xml_min and !$self->xml_min ) {
+	elsif ( $self->has_predicate ) {
 		$expect_one = 0;
 	}
 	else {
 		$expect_one = 1;
 	}
 
-	my $t_c = $self->type_constraint
-		or $self->error(
-		"No type constraint on attribute; did you specify 'isa'?",
-		       );
+	my $t_c = $self->type_constraint;
 
 	# check to see whether ArrayRef was specified
 	if ( $t_c->is_a_type_of("ArrayRef") ) {
@@ -132,13 +106,6 @@ method build_graph_node() {
 
 		$t_c = $t_c->type_parameter;
 	}
-	elsif ( $self->has_xml_max and $self->xml_max > 1 or
-			$self->has_xml_min and $self->xml_min > 1
-	       ) {
-		$self->error(
-	"min/max specified as >1, but type constraint is not an ArrayRef",
-		       );
-	}
 
 	# ok.  now let's walk the type constraint tree, and look for
 	# types
@@ -155,7 +122,7 @@ method build_graph_node() {
 			push @st, @{ $x->parents };
 		}
 		elsif ( $x->isa("Moose::Meta::TypeConstraint::Enum") ) {
-			push @st, $x->parent;
+			$expect_simple = 1;
 		}
 		elsif ( $x->isa("Moose::Meta::TypeConstraint::Role") ) {
 			# likely to be a wildcard.
@@ -181,9 +148,6 @@ method build_graph_node() {
 	my $node;
 	my $nodeName = $self->has_xml_nodeName ?
 		$self->xml_nodeName : $self->name;
-	my $nodeName_prefix = $self->has_xml_nodeName_prefix ?
-		$self->xml_nodeName_prefix : {};
-	my $nodeName_r_prefix = { reverse %$nodeName_prefix };
 
 	my $expect_concrete = ($expect_bool||0) +
 		($expect_simple||0) + @expect_type;
@@ -206,109 +170,63 @@ method build_graph_node() {
 		}
 	}
 
-	my $prefix_xx;
 	# plug-in type classes.
 	if ( @expect_role ) {
 		my @users = map { $_->name } types_of(@expect_role);
-		if ( $self->has_xml_nodeName and !ref $self->xml_nodeName ) {
-			$self->error(
-"Str value for xml_nodeName incompatible with specifying a role type "
-	."constraint"
-			);
-		}
 		$nodeName = {} if !ref $nodeName;
 		for my $user ( @users ) {
 			if ( $user->does("PRANG::Graph") ) {
-				my $plugin_nodeName = $user->root_element;
-				my $xmlns;
-				if ( $xmlns = eval { $user->xmlns }//"" ) {
-					if ( not exists $nodeName_r_prefix->{$xmlns} ) {
-						$prefix_xx ||= "a";
-						$prefix_xx++ while exists $nodeName_prefix->{$prefix_xx};
-						$nodeName_prefix->{$prefix_xx} = $xmlns;
-						$nodeName_r_prefix->{$xmlns} = $prefix_xx;
-					}
-					$plugin_nodeName = "$nodeName_r_prefix->{$xmlns}:$plugin_nodeName";
-				}
-				if ( exists $nodeName->{$plugin_nodeName} ) {
+				my $root_element = $user->root_element;
+				$nodeName->{$root_element} = $user;
+			}
+			elsif ( $user->does("PRANG::Graph::Class") ) {
+				if ( !$self->has_xml_nodeName_attr ) {
 					$self->error(
-"Both '$user' and '$nodeName->{$plugin_nodeName}' plug-in type specify nodename $plugin_nodeName".($xmlns ? " (xmlns $xmlns)" : "").", conflict",
+"Can't use role(s) @expect_role; no xml_nodeName_attr",
 					       );
 				}
-				$nodeName->{$plugin_nodeName} = $user;
 			}
 			else {
 				$self->error(
-"Can't use one or more of role(s) @expect_role; ".$user->name." needs to consume role PRANG::Graph (hint: did you forget to \"with 'PRANG::Graph';\"?)",
+"Can't use role(s) @expect_role; no mapping",
 					);
 			}
 			push @expect_type, $user;
-			$expect_concrete++;
 		}
 		$self->xml_nodeName({%$nodeName});
-		if ( !$self->has_xml_nodeName_prefix and keys %$nodeName_prefix ) {
-			$self->xml_nodeName_prefix($nodeName_prefix);
-		}
 	}
-	if (!$expect_concrete) {
-		$self->error(
-			"no type(s) specified (or, role evaluated to nothing)",
-		       )
-	}
-
-	if ( !ref $nodeName ) {
+	if ( !ref $nodeName and $expect_concrete ) {
 		my $expected = $expect_bool ? "Bool" :
 			$expect_simple ? "Str" : $expect_type[0];
 		$nodeName = { $nodeName => $expected };
-		$self->xml_nodeName($nodeName);
 	}
-
-	# we will be using 'delete' with nodeName, so copy it
-	$nodeName = {%$nodeName};
-
-	# figure out the XML namespace of this node and set it on the
-	# attribute
-	my %xmlns_opts;
-	if ( $self->has_xmlns ) {
-		$xmlns_opts{xmlns} = $self->xmlns;
+	elsif ( $expect_concrete ) {
+		$nodeName = { %$nodeName };
 	}
-	else {
-		my $xmlns = eval { $self->associated_class->name->xmlns } // "";
-		$xmlns_opts{xmlns} = $xmlns
-			if $xmlns;  # FIXME - should *always* set it!
-	}
-	if ( $self->has_xmlns_attr ) {
-		$xmlns_opts{xmlns_attr} = $self->xmlns_attr;
-	}
-	my $prefix_xmlns = sub {
-		my $name = shift;
-		if ( $nodeName_prefix and $name =~ /^(\w+):(\w+)/ ) {
-			my %this_xmlns_opts = %xmlns_opts;
-			my $xmlns = $nodeName_prefix->{$1}
-				or die "unknown prefix '$1' used on attribute ".$self->name." of ".eval{$self->associated_class->name};
-			$this_xmlns_opts{xmlns} = $xmlns;
-			($2, \%this_xmlns_opts);
-		}
-		else {
-			($name, \%xmlns_opts);
-		}
-	};
 
 	my @expect;
 	for my $class ( @expect_type ) {
-		my (@names) = grep { $nodeName->{$_} eq $class }
-			keys %$nodeName;
-
-		# auto-load the classes now... save problems later
-		if ( !eval{ $class->meta->can("marshall_in_element") } ) {
-			my $ok = eval "use $class; 1";
-			if ( !$ok ) {
-				die "problem auto-including class '$class'; (hint: did you expect '$class' to be a subtype, but forget to define it before it was used or not use BEGIN { } appropriately?); exception is: $@";
+		my @xmlns;
+		if ( $self->has_xmlns ) {
+			push @xmlns, (xmlns => $self->xmlns);
+		}
+		else {
+			my $xmlns = $self->associated_class->name->xmlns;
+			if ( !$class->can("xmlns") ) {
+				my $ok = eval "use $class; 1";
+				if ( !$ok ) {
+					die "problem auto-including class '$class'; exception is: $@";
+				}
+			}
+			if ( !$class->meta->can("marshall_in_element") ) {
+				die "'$class' can't marshall in; did you 'with' the 'PRANG::Graph::Class' role?";
+			}
+			if ( ($class->xmlns||"") ne ($xmlns||"") ) {
+				push @xmlns, (xmlns => $class->xmlns);
 			}
 		}
-		if ( !eval{ $class->meta->can("marshall_in_element") } ) {
-			die "'$class' can't marshall in; did you 'use PRANG::Graph'?";
-		}
+		my (@names) = grep { $nodeName->{$_} eq $class }
+			keys %$nodeName;
 
 		if ( !@names ) {
 			die "type '$class' specified as allowed on '"
@@ -323,13 +241,11 @@ method build_graph_node() {
 		}
 
 		for my $name ( @names ) {
-			my ($nn, $xmlns_args) =
-				$prefix_xmlns->($name);
 			push @expect, PRANG::Graph::Element->new(
-				%$xmlns_args,
+				@xmlns,
 				attrName => $self->name,
 				nodeClass => $class,
-				nodeName => $nn,
+				nodeName => $name,
 			       );
 			delete $nodeName->{$name};
 		}
@@ -343,12 +259,10 @@ method build_graph_node() {
 		# 'Bool' elements are a shorthand for the element
 		# 'maybe' being there.
 		for my $name ( @names ) {
-			my ($nn, $xmlns_args) = $prefix_xmlns->($name);
 			push @expect, PRANG::Graph::Element->new(
-				%$xmlns_args,
 				attrName => $self->name,
 				attIsArray => $expect_many,
-				nodeName => $nn,
+				nodeName => $name,
 			       );
 			delete $nodeName->{$name};
 		}
@@ -371,12 +285,9 @@ method build_graph_node() {
 			}
 			else {
 				# regular XML data style
-				my ($nn, $xmlns_args) =
-					$prefix_xmlns->($name);
 				push @expect, PRANG::Graph::Element->new(
-					%$xmlns_args,
 					attrName => $self->name,
-					nodeName => $nn,
+					nodeName => $name,
 					contents => PRANG::Graph::Text->new,
 				       );
 			}
@@ -384,102 +295,15 @@ method build_graph_node() {
 		}
 	}
 
-	# determine if we need explicit attributes to record the
-	# nodename and/or XML namespace.
-
-	# first rule.  If multiple prefix:nodeName entries map to the
-	# same type, then we would have an ambiguous type map, and
-	# therefore need at least one of name_attr and xmlns_attr
-	my $have_ambiguous;
-	my (%seen_types, %seen_xmlns, %seen_localname);
-	my $fixed_xmlns = $self->xmlns;
-	my $use_prefixes = $self->has_xml_nodeName_prefix;
-	if ( $fixed_xmlns and $use_prefixes ) {
-		$self->error("specify only one of 'xmlns' / 'xml_nodeName_prefix' (note: latter may be implied by use of roles)");
-	}
-	while ( my ($element_fullname, $class) = each %{$self->xml_nodeName}) {
-		my ($xmlns, $localname);
-		if ( $use_prefixes ) {
-			(my $prefix, $localname) =
-				($element_fullname =~ /^(?:(\w+):)?(\w+|\*)/);
-			$prefix //= "";
-			$xmlns = $nodeName_prefix->{$prefix}//"";
-		}
-		else {
-			$localname = $element_fullname;
-			$xmlns = $fixed_xmlns//"";
-		}
-
-		$localname //= "";
-		$seen_localname{$localname}++;
-		$seen_xmlns{$xmlns}++;
-
-		$have_ambiguous++ if $localname eq "*";
-		$have_ambiguous++ if $xmlns eq "*";
-
-		my $ent = [ $xmlns, $localname ];
-		if ( my $aref = $seen_types{$class} ) {
-			$have_ambiguous++;
-			push @$aref, $ent;
-		}
-		else {
-			$seen_types{$class} = [ $ent ];
-		}
-	}
-
-	# if all nodes have the same localname, we can use just
-	# xmlns_attr.  if all nodes have the same xmlns, we can use
-	# just name_attr
-	my @name_attr;
-	if ( $have_ambiguous ) {
-		if ( keys %seen_localname > 1 or $seen_localname{"*"} ) {
-			if ( !$self->has_xml_nodeName_attr ) {
-				$self->error(
-"xml_nodeName map ambiguities or wildcarding imply need for "
-	."xml_nodeName_attr, but none given",
-				       );
-			}
-			else {
-				my $attr = $self->xml_nodeName_attr;
-				push @name_attr, name_attr => $attr;
-				for my $x ( @expect ) {
-					$x->nodeName_attr($attr);
-				}
-			}
-		}
-		else {
-			push @name_attr, xml_nodeName => (keys %seen_localname)[0]//"";
-		}
-
-		if ( keys %seen_xmlns > 1 or $seen_xmlns{"*"} ) {
-			if ( !$self->has_xmlns_attr ) {
-				$self->error(
-"xml_nodeName map ambiguities or wildcarding imply need for "
-	."xmlns_attr, but none given",
-				       );
-			}
-			else {
-				my $attr = $self->xmlns_attr;
-				push @name_attr, xmlns_attr => $attr;
-				for my $x ( @expect ) {
-					$x->xmlns_attr($attr);
-				}
-			}
-		}
-		else {
-			push @name_attr, xmlns => (keys %seen_xmlns)[0]//"";
-		}
-	}
-	elsif ( $self->has_xmlns_attr or $self->has_xml_nodeName_attr ) {
-		$self->error("unnecessary use of xmlns_attr / xml_nodeName_attr");
-	}
-	elsif ( $self->has_xml_nodeName ) {
-		push @name_attr, type_map => {%{$self->xml_nodeName}};
-		if ( $self->has_xml_nodeName_prefix ) {
-			push @name_attr, type_map_prefix =>
-				{%{$self->xml_nodeName_prefix}};
-		}
-	}
+	# FIXME - sometimes, xml_nodeName_attr is not needed, and
+	# setting it breaks things - it's only needed if the nodeName
+	# map is ambiguous.
+	my @name_attr =
+		(($self->has_xml_nodeName_attr ? 
+			  ( name_attr => $self->xml_nodeName_attr ) : ()),
+		 (($self->has_xml_nodeName and ref $self->xml_nodeName) ?
+			  ( type_map => {%{$self->xml_nodeName}} ) : ()),
+		);
 
 	if ( @expect > 1 ) {
 		$node = PRANG::Graph::Choice->new(
@@ -498,12 +322,6 @@ method build_graph_node() {
 	if ( $expect_bool ) {
 		$expect_one = 0;
 	}
-	if ( $expect_one and !$expect_simple and
-		     !$self->is_required and !$self->has_default ) {
-		$self->warn_of(
-"expected element is not required, this can cause errors on marshall out"
-		       );
-	}
 
 	# deal with limits
 	if ( !$expect_one or $expect_many) {
@@ -559,7 +377,7 @@ PRANG::Graph::Meta::Element - metaclass metarole for XML elements
  # equivalent alternative - plays well with others!
  has 'somechild' =>
     is => "rw",
-    traits => [qw/PRANG::Element/],
+    traits => [qr/PRANG::Element/],
     isa => "Some::Type",
     xml_required => 0,
     ;
@@ -584,8 +402,8 @@ the type constraint, and also the predicate.
 If you like, you can also set the C<xmlns> and C<xml_nodeName>
 attribute property, to override the default behaviour, which is to
 assume that the XML element name matches the Moose attribute name, and
-that the XML namespace of the element is that of the enclosing class
-(ie, C<$class-E<gt>xmlns>), if defined.
+that the XML namespace of the element is that of the I<value> (ie,
+C<$object-E<gt>somechild-E<gt>xmlns>.
 
 The B<order> of declaring element attributes is important.  They
 implicitly define a "sequence".  To specify a "choice", you must use a
@@ -845,8 +663,8 @@ elements.
       isa => "ArrayRef[PRANG::XMLSchema::Whatever|Str]",
       xml_nodeName => { "" => "Str", "*" => "PRANG::XMLSchema::Whatever" },
       xml_nodeName_attr => "nodenames",
+      xml_nodeName_xmlns_attr => "nodenames_xmlns",
       xmlns => "*",
-      xmlns_attr => "nodenames_xmlns",
       ;
 
 B<FIXME:> this is currently unimplemented.
@@ -855,12 +673,42 @@ B<FIXME:> this is currently unimplemented.
 
 These are indicated by specifying a role.  At the time that the
 L<PRANG::Graph::Node> is built for the attribute, the currently
-available implementors of these roles are checked, which must all
-implement L<PRANG::Graph>.
+available implementors of these roles are checked, and depending on
+whether they implement L<PRANG::Graph> or merely
+L<PRANG::Graph::Class>, the following actions are taken:
 
-They Treated as if there is an C<xml_nodeName> entry for the class,
-from the C<root_element> value for the class to the type.  This allows
-writing extensible schemas.
+=over
+
+=item L<PRANG::Graph> types
+
+Treated as if there is an C<xml_nodeName> entry for the class, from
+the C<root_element> value for the class to the type.  B<FIXME>: to
+also include the XML namespace of the class.
+
+For writing extensible schemas, this is generally the role you want to
+inherit.
+
+=item L<PRANG::Graph::Class> types
+
+B<FIXME:> this entry may be out of date, please ignore for the time
+being.
+
+You must supply C<xml_nodeName_attr>.  This type will never be used on
+marshall in; however, it will happily work on the way out.
+
+This can be used when you have slots which may be unprocessed,
+L<PRANG::XMLSchema::Whatever> object structures, B<or> real
+L<PRANG::Graph::Class> instances.
+
+eg
+
+  has 'maybe_parsed' =>
+      is => "rw",
+      isa => "PRANG::Graph::Whatever|PRANG::Graph::Class",
+      xml_nodeName_attr => "maybe_parsed_name",
+      ;
+
+=back
 
 =back
 
