@@ -14,16 +14,6 @@ subtype "PRANG::XMLSchema::token"
 		!m{[\t\r\n]|^\s|\s$|\s\s};
 	};
 
-# automatically trim tokens if passed them.
-coerce "PRANG::XMLSchema::token"
-	=> from "Str",
-	=> via {
-		my ($x) = m/\A\s*(.*?)\s*\Z/;
-		$x =~ s{\s+}{ }g;
-		$x;
-	},
-	;
-
 # See https://rt.cpan.org/Ticket/Display.html?id=52309
 # use Regexp::Common qw/URI/;
 subtype "PRANG::XMLSchema::anyURI"
@@ -43,43 +33,12 @@ subtype "PRANG::XMLSchema::dateTime"
 	=> as "Str"
 	=> where {
 		# from the XMLSchema spec... it'll do for now ;)
-		# how on earth is one supposed to encode Pacific/Guam
-		# or Pacific/Saipan dates before 1845 with this regex?
 		m{
-^
 -?([1-9][0-9]{3,}|0[0-9]{3})
 -(0[1-9]|1[0-2])
 -(0[1-9]|[12][0-9]|3[01])
 T(([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9](\.[0-9]+)?|(24:00:00(\.0+)?))
 (?:Z|(?:\+|-)(?:(?:0[0-9]|1[0-3]):[0-5][0-9]|14:00))?
-$
-	 }x;
-	};
-
-subtype "PRANG::XMLSchema::time"
-	=> as "Str"
-	=> where {
-		# from the XMLSchema spec... it'll do for now ;)
-		m{
-^
-(([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9](\.[0-9]+)?|(24:00:00(\.0+)?))
-(?:Z|(?:\+|-)(?:(?:0[0-9]|1[0-3]):[0-5][0-9]|14:00))?
-$
-	 }x;
-	};
-
-subtype "PRANG::XMLSchema::date"
-	=> as "Str"
-	=> where {
-		# from the XMLSchema spec... it'll do for now ;)
-		# XXX: Note, since the XML Spec bizarrely has Timezones on Dates,
-		# we have chosen to ignore it (since it is optional anyway)
-		m{
-^
--?([1-9][0-9]{3,}|0[0-9]{3})
--(0[1-9]|1[0-2])
--(0[1-9]|[12][0-9]|3[01])
-$
 	 }x;
 	};
 
@@ -104,15 +63,9 @@ subtype "PRANG::XMLSchema::boolean"
 	=> where {
 		m/^(?:0|1|true|false)$/;
 	};
-
 coerce "Bool"
 	=> from 'PRANG::XMLSchema::boolean'
 	=> via { m{1|true} ? 1 : 0 };
-
-coerce "PRANG::XMLSchema::boolean"
-	=> from 'Bool'
-	=> via { (!!$_) ? "true" : "false" };
-
 subtype "PRANG::XMLSchema::decimal"
 	=> as "Num";
 
